// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: pinba.proto

#ifndef PROTOBUF_pinba_2eproto__INCLUDED
#define PROTOBUF_pinba_2eproto__INCLUDED

#include <string>

#include <google/protobuf/stubs/common.h>

#if GOOGLE_PROTOBUF_VERSION < 2004000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers.  Please update
#error your headers.
#endif
#if 2004001 < GOOGLE_PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers.  Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/repeated_field.h>
#include <google/protobuf/extension_set.h>
// @@protoc_insertion_point(includes)

namespace Pinba {

// Internal implementation detail -- do not call these.
void  protobuf_AddDesc_pinba_2eproto();
void protobuf_AssignDesc_pinba_2eproto();
void protobuf_ShutdownFile_pinba_2eproto();

class Request;

// ===================================================================

class Request : public ::google::protobuf::MessageLite {
 public:
  Request();
  virtual ~Request();
  
  Request(const Request& from);
  
  inline Request& operator=(const Request& from) {
    CopyFrom(from);
    return *this;
  }
  
  static const Request& default_instance();
  
  void Swap(Request* other);
  
  // implements Message ----------------------------------------------
  
  Request* New() const;
  void CheckTypeAndMergeFrom(const ::google::protobuf::MessageLite& from);
  void CopyFrom(const Request& from);
  void MergeFrom(const Request& from);
  void Clear();
  bool IsInitialized() const;
  
  int ByteSize() const;
  bool MergePartialFromCodedStream(
      ::google::protobuf::io::CodedInputStream* input);
  void SerializeWithCachedSizes(
      ::google::protobuf::io::CodedOutputStream* output) const;
  int GetCachedSize() const { return _cached_size_; }
  private:
  void SharedCtor();
  void SharedDtor();
  void SetCachedSize(int size) const;
  public:
  
  ::std::string GetTypeName() const;
  
  // nested types ----------------------------------------------------
  
  // accessors -------------------------------------------------------
  
  // required string hostname = 1;
  inline bool has_hostname() const;
  inline void clear_hostname();
  static const int kHostnameFieldNumber = 1;
  inline const ::std::string& hostname() const;
  inline void set_hostname(const ::std::string& value);
  inline void set_hostname(const char* value);
  inline void set_hostname(const char* value, size_t size);
  inline ::std::string* mutable_hostname();
  inline ::std::string* release_hostname();
  
  // required string server_name = 2;
  inline bool has_server_name() const;
  inline void clear_server_name();
  static const int kServerNameFieldNumber = 2;
  inline const ::std::string& server_name() const;
  inline void set_server_name(const ::std::string& value);
  inline void set_server_name(const char* value);
  inline void set_server_name(const char* value, size_t size);
  inline ::std::string* mutable_server_name();
  inline ::std::string* release_server_name();
  
  // required string script_name = 3;
  inline bool has_script_name() const;
  inline void clear_script_name();
  static const int kScriptNameFieldNumber = 3;
  inline const ::std::string& script_name() const;
  inline void set_script_name(const ::std::string& value);
  inline void set_script_name(const char* value);
  inline void set_script_name(const char* value, size_t size);
  inline ::std::string* mutable_script_name();
  inline ::std::string* release_script_name();
  
  // required uint32 request_count = 4;
  inline bool has_request_count() const;
  inline void clear_request_count();
  static const int kRequestCountFieldNumber = 4;
  inline ::google::protobuf::uint32 request_count() const;
  inline void set_request_count(::google::protobuf::uint32 value);
  
  // required uint32 document_size = 5;
  inline bool has_document_size() const;
  inline void clear_document_size();
  static const int kDocumentSizeFieldNumber = 5;
  inline ::google::protobuf::uint32 document_size() const;
  inline void set_document_size(::google::protobuf::uint32 value);
  
  // required uint32 memory_peak = 6;
  inline bool has_memory_peak() const;
  inline void clear_memory_peak();
  static const int kMemoryPeakFieldNumber = 6;
  inline ::google::protobuf::uint32 memory_peak() const;
  inline void set_memory_peak(::google::protobuf::uint32 value);
  
  // required float request_time = 7;
  inline bool has_request_time() const;
  inline void clear_request_time();
  static const int kRequestTimeFieldNumber = 7;
  inline float request_time() const;
  inline void set_request_time(float value);
  
  // required float ru_utime = 8;
  inline bool has_ru_utime() const;
  inline void clear_ru_utime();
  static const int kRuUtimeFieldNumber = 8;
  inline float ru_utime() const;
  inline void set_ru_utime(float value);
  
  // required float ru_stime = 9;
  inline bool has_ru_stime() const;
  inline void clear_ru_stime();
  static const int kRuStimeFieldNumber = 9;
  inline float ru_stime() const;
  inline void set_ru_stime(float value);
  
  // repeated uint32 timer_hit_count = 10;
  inline int timer_hit_count_size() const;
  inline void clear_timer_hit_count();
  static const int kTimerHitCountFieldNumber = 10;
  inline ::google::protobuf::uint32 timer_hit_count(int index) const;
  inline void set_timer_hit_count(int index, ::google::protobuf::uint32 value);
  inline void add_timer_hit_count(::google::protobuf::uint32 value);
  inline const ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >&
      timer_hit_count() const;
  inline ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >*
      mutable_timer_hit_count();
  
  // repeated float timer_value = 11;
  inline int timer_value_size() const;
  inline void clear_timer_value();
  static const int kTimerValueFieldNumber = 11;
  inline float timer_value(int index) const;
  inline void set_timer_value(int index, float value);
  inline void add_timer_value(float value);
  inline const ::google::protobuf::RepeatedField< float >&
      timer_value() const;
  inline ::google::protobuf::RepeatedField< float >*
      mutable_timer_value();
  
  // repeated uint32 timer_tag_count = 12;
  inline int timer_tag_count_size() const;
  inline void clear_timer_tag_count();
  static const int kTimerTagCountFieldNumber = 12;
  inline ::google::protobuf::uint32 timer_tag_count(int index) const;
  inline void set_timer_tag_count(int index, ::google::protobuf::uint32 value);
  inline void add_timer_tag_count(::google::protobuf::uint32 value);
  inline const ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >&
      timer_tag_count() const;
  inline ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >*
      mutable_timer_tag_count();
  
  // repeated uint32 timer_tag_name = 13;
  inline int timer_tag_name_size() const;
  inline void clear_timer_tag_name();
  static const int kTimerTagNameFieldNumber = 13;
  inline ::google::protobuf::uint32 timer_tag_name(int index) const;
  inline void set_timer_tag_name(int index, ::google::protobuf::uint32 value);
  inline void add_timer_tag_name(::google::protobuf::uint32 value);
  inline const ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >&
      timer_tag_name() const;
  inline ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >*
      mutable_timer_tag_name();
  
  // repeated uint32 timer_tag_value = 14;
  inline int timer_tag_value_size() const;
  inline void clear_timer_tag_value();
  static const int kTimerTagValueFieldNumber = 14;
  inline ::google::protobuf::uint32 timer_tag_value(int index) const;
  inline void set_timer_tag_value(int index, ::google::protobuf::uint32 value);
  inline void add_timer_tag_value(::google::protobuf::uint32 value);
  inline const ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >&
      timer_tag_value() const;
  inline ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >*
      mutable_timer_tag_value();
  
  // repeated string dictionary = 15;
  inline int dictionary_size() const;
  inline void clear_dictionary();
  static const int kDictionaryFieldNumber = 15;
  inline const ::std::string& dictionary(int index) const;
  inline ::std::string* mutable_dictionary(int index);
  inline void set_dictionary(int index, const ::std::string& value);
  inline void set_dictionary(int index, const char* value);
  inline void set_dictionary(int index, const char* value, size_t size);
  inline ::std::string* add_dictionary();
  inline void add_dictionary(const ::std::string& value);
  inline void add_dictionary(const char* value);
  inline void add_dictionary(const char* value, size_t size);
  inline const ::google::protobuf::RepeatedPtrField< ::std::string>& dictionary() const;
  inline ::google::protobuf::RepeatedPtrField< ::std::string>* mutable_dictionary();
  
  // optional uint32 status = 16;
  inline bool has_status() const;
  inline void clear_status();
  static const int kStatusFieldNumber = 16;
  inline ::google::protobuf::uint32 status() const;
  inline void set_status(::google::protobuf::uint32 value);
  
  // @@protoc_insertion_point(class_scope:Pinba.Request)
 private:
  inline void set_has_hostname();
  inline void clear_has_hostname();
  inline void set_has_server_name();
  inline void clear_has_server_name();
  inline void set_has_script_name();
  inline void clear_has_script_name();
  inline void set_has_request_count();
  inline void clear_has_request_count();
  inline void set_has_document_size();
  inline void clear_has_document_size();
  inline void set_has_memory_peak();
  inline void clear_has_memory_peak();
  inline void set_has_request_time();
  inline void clear_has_request_time();
  inline void set_has_ru_utime();
  inline void clear_has_ru_utime();
  inline void set_has_ru_stime();
  inline void clear_has_ru_stime();
  inline void set_has_status();
  inline void clear_has_status();
  
  ::std::string* hostname_;
  ::std::string* server_name_;
  ::std::string* script_name_;
  ::google::protobuf::uint32 request_count_;
  ::google::protobuf::uint32 document_size_;
  ::google::protobuf::uint32 memory_peak_;
  float request_time_;
  float ru_utime_;
  float ru_stime_;
  ::google::protobuf::RepeatedField< ::google::protobuf::uint32 > timer_hit_count_;
  ::google::protobuf::RepeatedField< float > timer_value_;
  ::google::protobuf::RepeatedField< ::google::protobuf::uint32 > timer_tag_count_;
  ::google::protobuf::RepeatedField< ::google::protobuf::uint32 > timer_tag_name_;
  ::google::protobuf::RepeatedField< ::google::protobuf::uint32 > timer_tag_value_;
  ::google::protobuf::RepeatedPtrField< ::std::string> dictionary_;
  ::google::protobuf::uint32 status_;
  
  mutable int _cached_size_;
  ::google::protobuf::uint32 _has_bits_[(16 + 31) / 32];
  
  friend void  protobuf_AddDesc_pinba_2eproto();
  friend void protobuf_AssignDesc_pinba_2eproto();
  friend void protobuf_ShutdownFile_pinba_2eproto();
  
  void InitAsDefaultInstance();
  static Request* default_instance_;
};
// ===================================================================


// ===================================================================

// Request

// required string hostname = 1;
inline bool Request::has_hostname() const {
  return (_has_bits_[0] & 0x00000001u) != 0;
}
inline void Request::set_has_hostname() {
  _has_bits_[0] |= 0x00000001u;
}
inline void Request::clear_has_hostname() {
  _has_bits_[0] &= ~0x00000001u;
}
inline void Request::clear_hostname() {
  if (hostname_ != &::google::protobuf::internal::kEmptyString) {
    hostname_->clear();
  }
  clear_has_hostname();
}
inline const ::std::string& Request::hostname() const {
  return *hostname_;
}
inline void Request::set_hostname(const ::std::string& value) {
  set_has_hostname();
  if (hostname_ == &::google::protobuf::internal::kEmptyString) {
    hostname_ = new ::std::string;
  }
  hostname_->assign(value);
}
inline void Request::set_hostname(const char* value) {
  set_has_hostname();
  if (hostname_ == &::google::protobuf::internal::kEmptyString) {
    hostname_ = new ::std::string;
  }
  hostname_->assign(value);
}
inline void Request::set_hostname(const char* value, size_t size) {
  set_has_hostname();
  if (hostname_ == &::google::protobuf::internal::kEmptyString) {
    hostname_ = new ::std::string;
  }
  hostname_->assign(reinterpret_cast<const char*>(value), size);
}
inline ::std::string* Request::mutable_hostname() {
  set_has_hostname();
  if (hostname_ == &::google::protobuf::internal::kEmptyString) {
    hostname_ = new ::std::string;
  }
  return hostname_;
}
inline ::std::string* Request::release_hostname() {
  clear_has_hostname();
  if (hostname_ == &::google::protobuf::internal::kEmptyString) {
    return NULL;
  } else {
    ::std::string* temp = hostname_;
    hostname_ = const_cast< ::std::string*>(&::google::protobuf::internal::kEmptyString);
    return temp;
  }
}

// required string server_name = 2;
inline bool Request::has_server_name() const {
  return (_has_bits_[0] & 0x00000002u) != 0;
}
inline void Request::set_has_server_name() {
  _has_bits_[0] |= 0x00000002u;
}
inline void Request::clear_has_server_name() {
  _has_bits_[0] &= ~0x00000002u;
}
inline void Request::clear_server_name() {
  if (server_name_ != &::google::protobuf::internal::kEmptyString) {
    server_name_->clear();
  }
  clear_has_server_name();
}
inline const ::std::string& Request::server_name() const {
  return *server_name_;
}
inline void Request::set_server_name(const ::std::string& value) {
  set_has_server_name();
  if (server_name_ == &::google::protobuf::internal::kEmptyString) {
    server_name_ = new ::std::string;
  }
  server_name_->assign(value);
}
inline void Request::set_server_name(const char* value) {
  set_has_server_name();
  if (server_name_ == &::google::protobuf::internal::kEmptyString) {
    server_name_ = new ::std::string;
  }
  server_name_->assign(value);
}
inline void Request::set_server_name(const char* value, size_t size) {
  set_has_server_name();
  if (server_name_ == &::google::protobuf::internal::kEmptyString) {
    server_name_ = new ::std::string;
  }
  server_name_->assign(reinterpret_cast<const char*>(value), size);
}
inline ::std::string* Request::mutable_server_name() {
  set_has_server_name();
  if (server_name_ == &::google::protobuf::internal::kEmptyString) {
    server_name_ = new ::std::string;
  }
  return server_name_;
}
inline ::std::string* Request::release_server_name() {
  clear_has_server_name();
  if (server_name_ == &::google::protobuf::internal::kEmptyString) {
    return NULL;
  } else {
    ::std::string* temp = server_name_;
    server_name_ = const_cast< ::std::string*>(&::google::protobuf::internal::kEmptyString);
    return temp;
  }
}

// required string script_name = 3;
inline bool Request::has_script_name() const {
  return (_has_bits_[0] & 0x00000004u) != 0;
}
inline void Request::set_has_script_name() {
  _has_bits_[0] |= 0x00000004u;
}
inline void Request::clear_has_script_name() {
  _has_bits_[0] &= ~0x00000004u;
}
inline void Request::clear_script_name() {
  if (script_name_ != &::google::protobuf::internal::kEmptyString) {
    script_name_->clear();
  }
  clear_has_script_name();
}
inline const ::std::string& Request::script_name() const {
  return *script_name_;
}
inline void Request::set_script_name(const ::std::string& value) {
  set_has_script_name();
  if (script_name_ == &::google::protobuf::internal::kEmptyString) {
    script_name_ = new ::std::string;
  }
  script_name_->assign(value);
}
inline void Request::set_script_name(const char* value) {
  set_has_script_name();
  if (script_name_ == &::google::protobuf::internal::kEmptyString) {
    script_name_ = new ::std::string;
  }
  script_name_->assign(value);
}
inline void Request::set_script_name(const char* value, size_t size) {
  set_has_script_name();
  if (script_name_ == &::google::protobuf::internal::kEmptyString) {
    script_name_ = new ::std::string;
  }
  script_name_->assign(reinterpret_cast<const char*>(value), size);
}
inline ::std::string* Request::mutable_script_name() {
  set_has_script_name();
  if (script_name_ == &::google::protobuf::internal::kEmptyString) {
    script_name_ = new ::std::string;
  }
  return script_name_;
}
inline ::std::string* Request::release_script_name() {
  clear_has_script_name();
  if (script_name_ == &::google::protobuf::internal::kEmptyString) {
    return NULL;
  } else {
    ::std::string* temp = script_name_;
    script_name_ = const_cast< ::std::string*>(&::google::protobuf::internal::kEmptyString);
    return temp;
  }
}

// required uint32 request_count = 4;
inline bool Request::has_request_count() const {
  return (_has_bits_[0] & 0x00000008u) != 0;
}
inline void Request::set_has_request_count() {
  _has_bits_[0] |= 0x00000008u;
}
inline void Request::clear_has_request_count() {
  _has_bits_[0] &= ~0x00000008u;
}
inline void Request::clear_request_count() {
  request_count_ = 0u;
  clear_has_request_count();
}
inline ::google::protobuf::uint32 Request::request_count() const {
  return request_count_;
}
inline void Request::set_request_count(::google::protobuf::uint32 value) {
  set_has_request_count();
  request_count_ = value;
}

// required uint32 document_size = 5;
inline bool Request::has_document_size() const {
  return (_has_bits_[0] & 0x00000010u) != 0;
}
inline void Request::set_has_document_size() {
  _has_bits_[0] |= 0x00000010u;
}
inline void Request::clear_has_document_size() {
  _has_bits_[0] &= ~0x00000010u;
}
inline void Request::clear_document_size() {
  document_size_ = 0u;
  clear_has_document_size();
}
inline ::google::protobuf::uint32 Request::document_size() const {
  return document_size_;
}
inline void Request::set_document_size(::google::protobuf::uint32 value) {
  set_has_document_size();
  document_size_ = value;
}

// required uint32 memory_peak = 6;
inline bool Request::has_memory_peak() const {
  return (_has_bits_[0] & 0x00000020u) != 0;
}
inline void Request::set_has_memory_peak() {
  _has_bits_[0] |= 0x00000020u;
}
inline void Request::clear_has_memory_peak() {
  _has_bits_[0] &= ~0x00000020u;
}
inline void Request::clear_memory_peak() {
  memory_peak_ = 0u;
  clear_has_memory_peak();
}
inline ::google::protobuf::uint32 Request::memory_peak() const {
  return memory_peak_;
}
inline void Request::set_memory_peak(::google::protobuf::uint32 value) {
  set_has_memory_peak();
  memory_peak_ = value;
}

// required float request_time = 7;
inline bool Request::has_request_time() const {
  return (_has_bits_[0] & 0x00000040u) != 0;
}
inline void Request::set_has_request_time() {
  _has_bits_[0] |= 0x00000040u;
}
inline void Request::clear_has_request_time() {
  _has_bits_[0] &= ~0x00000040u;
}
inline void Request::clear_request_time() {
  request_time_ = 0;
  clear_has_request_time();
}
inline float Request::request_time() const {
  return request_time_;
}
inline void Request::set_request_time(float value) {
  set_has_request_time();
  request_time_ = value;
}

// required float ru_utime = 8;
inline bool Request::has_ru_utime() const {
  return (_has_bits_[0] & 0x00000080u) != 0;
}
inline void Request::set_has_ru_utime() {
  _has_bits_[0] |= 0x00000080u;
}
inline void Request::clear_has_ru_utime() {
  _has_bits_[0] &= ~0x00000080u;
}
inline void Request::clear_ru_utime() {
  ru_utime_ = 0;
  clear_has_ru_utime();
}
inline float Request::ru_utime() const {
  return ru_utime_;
}
inline void Request::set_ru_utime(float value) {
  set_has_ru_utime();
  ru_utime_ = value;
}

// required float ru_stime = 9;
inline bool Request::has_ru_stime() const {
  return (_has_bits_[0] & 0x00000100u) != 0;
}
inline void Request::set_has_ru_stime() {
  _has_bits_[0] |= 0x00000100u;
}
inline void Request::clear_has_ru_stime() {
  _has_bits_[0] &= ~0x00000100u;
}
inline void Request::clear_ru_stime() {
  ru_stime_ = 0;
  clear_has_ru_stime();
}
inline float Request::ru_stime() const {
  return ru_stime_;
}
inline void Request::set_ru_stime(float value) {
  set_has_ru_stime();
  ru_stime_ = value;
}

// repeated uint32 timer_hit_count = 10;
inline int Request::timer_hit_count_size() const {
  return timer_hit_count_.size();
}
inline void Request::clear_timer_hit_count() {
  timer_hit_count_.Clear();
}
inline ::google::protobuf::uint32 Request::timer_hit_count(int index) const {
  return timer_hit_count_.Get(index);
}
inline void Request::set_timer_hit_count(int index, ::google::protobuf::uint32 value) {
  timer_hit_count_.Set(index, value);
}
inline void Request::add_timer_hit_count(::google::protobuf::uint32 value) {
  timer_hit_count_.Add(value);
}
inline const ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >&
Request::timer_hit_count() const {
  return timer_hit_count_;
}
inline ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >*
Request::mutable_timer_hit_count() {
  return &timer_hit_count_;
}

// repeated float timer_value = 11;
inline int Request::timer_value_size() const {
  return timer_value_.size();
}
inline void Request::clear_timer_value() {
  timer_value_.Clear();
}
inline float Request::timer_value(int index) const {
  return timer_value_.Get(index);
}
inline void Request::set_timer_value(int index, float value) {
  timer_value_.Set(index, value);
}
inline void Request::add_timer_value(float value) {
  timer_value_.Add(value);
}
inline const ::google::protobuf::RepeatedField< float >&
Request::timer_value() const {
  return timer_value_;
}
inline ::google::protobuf::RepeatedField< float >*
Request::mutable_timer_value() {
  return &timer_value_;
}

// repeated uint32 timer_tag_count = 12;
inline int Request::timer_tag_count_size() const {
  return timer_tag_count_.size();
}
inline void Request::clear_timer_tag_count() {
  timer_tag_count_.Clear();
}
inline ::google::protobuf::uint32 Request::timer_tag_count(int index) const {
  return timer_tag_count_.Get(index);
}
inline void Request::set_timer_tag_count(int index, ::google::protobuf::uint32 value) {
  timer_tag_count_.Set(index, value);
}
inline void Request::add_timer_tag_count(::google::protobuf::uint32 value) {
  timer_tag_count_.Add(value);
}
inline const ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >&
Request::timer_tag_count() const {
  return timer_tag_count_;
}
inline ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >*
Request::mutable_timer_tag_count() {
  return &timer_tag_count_;
}

// repeated uint32 timer_tag_name = 13;
inline int Request::timer_tag_name_size() const {
  return timer_tag_name_.size();
}
inline void Request::clear_timer_tag_name() {
  timer_tag_name_.Clear();
}
inline ::google::protobuf::uint32 Request::timer_tag_name(int index) const {
  return timer_tag_name_.Get(index);
}
inline void Request::set_timer_tag_name(int index, ::google::protobuf::uint32 value) {
  timer_tag_name_.Set(index, value);
}
inline void Request::add_timer_tag_name(::google::protobuf::uint32 value) {
  timer_tag_name_.Add(value);
}
inline const ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >&
Request::timer_tag_name() const {
  return timer_tag_name_;
}
inline ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >*
Request::mutable_timer_tag_name() {
  return &timer_tag_name_;
}

// repeated uint32 timer_tag_value = 14;
inline int Request::timer_tag_value_size() const {
  return timer_tag_value_.size();
}
inline void Request::clear_timer_tag_value() {
  timer_tag_value_.Clear();
}
inline ::google::protobuf::uint32 Request::timer_tag_value(int index) const {
  return timer_tag_value_.Get(index);
}
inline void Request::set_timer_tag_value(int index, ::google::protobuf::uint32 value) {
  timer_tag_value_.Set(index, value);
}
inline void Request::add_timer_tag_value(::google::protobuf::uint32 value) {
  timer_tag_value_.Add(value);
}
inline const ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >&
Request::timer_tag_value() const {
  return timer_tag_value_;
}
inline ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >*
Request::mutable_timer_tag_value() {
  return &timer_tag_value_;
}

// repeated string dictionary = 15;
inline int Request::dictionary_size() const {
  return dictionary_.size();
}
inline void Request::clear_dictionary() {
  dictionary_.Clear();
}
inline const ::std::string& Request::dictionary(int index) const {
  return dictionary_.Get(index);
}
inline ::std::string* Request::mutable_dictionary(int index) {
  return dictionary_.Mutable(index);
}
inline void Request::set_dictionary(int index, const ::std::string& value) {
  dictionary_.Mutable(index)->assign(value);
}
inline void Request::set_dictionary(int index, const char* value) {
  dictionary_.Mutable(index)->assign(value);
}
inline void Request::set_dictionary(int index, const char* value, size_t size) {
  dictionary_.Mutable(index)->assign(
    reinterpret_cast<const char*>(value), size);
}
inline ::std::string* Request::add_dictionary() {
  return dictionary_.Add();
}
inline void Request::add_dictionary(const ::std::string& value) {
  dictionary_.Add()->assign(value);
}
inline void Request::add_dictionary(const char* value) {
  dictionary_.Add()->assign(value);
}
inline void Request::add_dictionary(const char* value, size_t size) {
  dictionary_.Add()->assign(reinterpret_cast<const char*>(value), size);
}
inline const ::google::protobuf::RepeatedPtrField< ::std::string>&
Request::dictionary() const {
  return dictionary_;
}
inline ::google::protobuf::RepeatedPtrField< ::std::string>*
Request::mutable_dictionary() {
  return &dictionary_;
}

// optional uint32 status = 16;
inline bool Request::has_status() const {
  return (_has_bits_[0] & 0x00008000u) != 0;
}
inline void Request::set_has_status() {
  _has_bits_[0] |= 0x00008000u;
}
inline void Request::clear_has_status() {
  _has_bits_[0] &= ~0x00008000u;
}
inline void Request::clear_status() {
  status_ = 0u;
  clear_has_status();
}
inline ::google::protobuf::uint32 Request::status() const {
  return status_;
}
inline void Request::set_status(::google::protobuf::uint32 value) {
  set_has_status();
  status_ = value;
}


// @@protoc_insertion_point(namespace_scope)

}  // namespace Pinba

// @@protoc_insertion_point(global_scope)

#endif  // PROTOBUF_pinba_2eproto__INCLUDED
